﻿//------------------------------------------------------------------------------
// IdeaSparx.CoolControls.Web
// Author: John Eric Sobrepena (2009)
// You can use these codes in whole or in parts without warranty.
// By using any part of this code, you agree 
// to keep this information about the author intact.
// http://johnsobrepena.blogspot.com
//------------------------------------------------------------------------------

using System;
using System.Collections.Generic;
using System.Web;
using System.Web.UI.WebControls;
using System.ComponentModel;
using System.Web.UI;
using System.Drawing;
using System.Text.RegularExpressions;
using System.Text;
using System.IO;

[assembly: WebResource("IdeaSparx.CoolControls.Web.CoolGridView.js", "application/x-javascript")]
[assembly: WebResource("IdeaSparx.CoolControls.Web.CoolCore.js", "application/x-javascript")]
namespace IdeaSparx.CoolControls.Web
{
    [ToolboxData("<{0}:CoolGridView Width=\"400px\" Height=\"300px\"></{0}:CoolGridView>"),
    Designer(typeof(CoolGridViewDesigner))]
    public class CoolGridView : GridView
    {
        private const string Suffix = "jEsCoOl";
        private CoolGridViewRow PagerRow = null;
        private string _HiddenFieldDataValue = String.Empty;

        private bool _FixHeaders = false;
        [Browsable(true), Category("Behavior"), DefaultValue(false)]
        public bool FixHeaders
        {
            get { return _FixHeaders; }
            set { _FixHeaders = value; }
        }

        private CoolGridViewViewMode _ViewMode = CoolGridViewViewMode.Flat;
        /// <summary>
        /// Get or set the ViewMode either Hierarchical or Flat. Flat view is the default.
        /// </summary>
        [Browsable(true), Category("Behavior"), DefaultValue(CoolGridViewViewMode.Flat)]
        public CoolGridViewViewMode ViewMode
        {
            get { return _ViewMode; }
            set { _ViewMode = value; }
        }

        private bool _AllowResizeColumn = true;
        /// <summary>
        /// Get or set if user can resize the column.
        /// </summary>
        [Browsable(true), Category("Behavior"), DefaultValue(true)]
        public bool AllowResizeColumn
        {
            get { return _AllowResizeColumn; }
            set { _AllowResizeColumn = value; }
        }

        private Unit _DefaultColumnWidth = new Unit(100,UnitType.Pixel);
        [Browsable(true), Category("Appearance"), DefaultValue(typeof(Unit),"100px")]
        public Unit DefaultColumnWidth
        {
            get { return _DefaultColumnWidth; }
            set {
                if (value == Unit.Empty)
                    throw new ArgumentException("DefaultColumnWidth cannot be empty.");
                if (value.Type != UnitType.Pixel)
                    throw new ArgumentException("DefaultColumnWidth can only be of type Pixel");
                _DefaultColumnWidth = value; 
            }
        }

        //The template of the child-control
        private ITemplate _ChildTemplate;
        [TemplateContainer(typeof(CoolGridView)), 
        Description("Child template"), 
        Browsable(false), 
        DefaultValue(null), 
        PersistenceMode(PersistenceMode.InnerProperty)]
        public ITemplate ChildTemplate
        {
            get { return _ChildTemplate; }
            set { _ChildTemplate = value; }
        }

        internal CoolGridViewChildHost _ChildInstance = null;

        protected override int CreateChildControls(System.Collections.IEnumerable dataSource, bool dataBinding)
        {
            int r = base.CreateChildControls(dataSource, dataBinding);

            if (ChildTemplate != null)
            {
                CoolGridViewChildHost host = new CoolGridViewChildHost();
                ChildTemplate.InstantiateIn(host);
                _ChildInstance = host;
                this.Controls.Add(host);
            }
            return r;
        }

        #region Parsers
        public List<Unit> ParseColumnWidthsFromJson(string JsonString)
        {
            List<Unit> columnWidths = new List<Unit>();
            if (!String.IsNullOrEmpty(JsonString))
            {

                RegexOptions options = (RegexOptions.IgnorePatternWhitespace | RegexOptions.Singleline | RegexOptions.IgnoreCase);
                Regex rx = new Regex("ColumnWidths\\s*:\\s*[[](?:([0-9.]+) || [,\\s]*)*[]]", options);

                MatchCollection mc = rx.Matches(JsonString);
                if (mc.Count > 0 && mc[0].Groups.Count > 1)
                    foreach (Capture c in mc[0].Groups[1].Captures)
                    {
                        columnWidths.Add(Unit.Parse(c.Value + "px"));
                    }
            }

            return columnWidths;
        }
        #endregion

        //Constructor
        public CoolGridView()
        {
            _BoundaryStyle.BorderColor = Color.Gray;
            _BoundaryStyle.BorderWidth = new Unit(1, UnitType.Pixel);
            _BoundaryStyle.BorderStyle = BorderStyle.Solid;
        }

        protected override GridViewRow CreateRow(int rowIndex, int dataSourceIndex, DataControlRowType rowType, DataControlRowState rowState)
        {
            if (rowType == DataControlRowType.Header
                || rowType == DataControlRowType.Footer
                || rowType == DataControlRowType.Pager
                //|| rowType == DataControlRowType.EmptyDataRow
                )
            {
                CoolGridViewRow c = new CoolGridViewRow(rowIndex, dataSourceIndex, rowType, rowState);
                c.ParentCoolGridView = this;
                if (rowType == DataControlRowType.Pager) PagerRow = c;
                return c;
            }
            else if (rowIndex == 0)
            {
                CoolGridViewFirstDataRow c = new CoolGridViewFirstDataRow(rowIndex, dataSourceIndex, rowType, rowState);
                c.ParentCoolGridView = this;
                return c;
            }
            else
            {
                return new CoolGridViewDataRow(rowIndex, dataSourceIndex, rowType, rowState);
            }
        }

        private Unit _Width = Unit.Empty;
        [DefaultValue(typeof(Unit), ""), Category("Layout")]
        public override Unit Width
        {
            get { return _Width; }
            set { _Width = value; }
        }

        private Unit _Height = Unit.Empty;
        [DefaultValue(typeof(Unit), ""), Category("Layout")]
        public override Unit Height
        {
            get { return _Height; }
            set { _Height = value; }
        }

        private new bool EnableSortingAndPagingCallbacks
        {
            get { return false; }
            set { }
        }

        private Style _BoundaryStyle = new Style();
        [PersistenceMode(PersistenceMode.InnerProperty), Description("Boundary Style"), NotifyParentProperty(true),
        Category("Styles"), DesignerSerializationVisibility(DesignerSerializationVisibility.Content), Browsable(true)]
        public Style BoundaryStyle
        {
            get { return _BoundaryStyle; }
        }

        private string GridContainerID { get { return ClientID + Suffix + "_mainDiv"; } }
        private string HeaderContainerID { get { return ClientID + Suffix + "_headerDiv"; } }
        private string FooterContainerID { get { return ClientID + Suffix + "_footerDiv"; } }
        private string TableContainerID { get { return ClientID + Suffix + "_tableDiv"; } }
        private string PagerContainerID { get { return ClientID + Suffix + "_pagerDiv"; } }
        private string HiddenFieldDataID { get { return ClientID + Suffix + "_data"; } }

        protected override void OnPreRender(EventArgs e)
        {
            //            //TODO: Blog this
            //            //Updated for AjaxControlToolkit compatibility
            //            string jsInitialize = @"var var{3} = new CoolGridView({{GridContainerID: ""{0}"",HeaderContainerID: ""{1}"",TableContainerID: ""{2}"",GridID: ""{3}"",FooterContainerID: ""{4}"",PagerContainerID: ""{5}""}});
            ////this is for AjaxControlToolkit compatibility
            //try{{Sys.WebForms.PageRequestManager.getInstance().add_endRequest(var{3}.GetAjaxToolkitEndRequestHander());}} catch(e){{}}";

            //            jsInitialize = String.Format(jsInitialize, GridContainerID, HeaderContainerID, TableContainerID, ClientID, FooterContainerID, PagerContainerID);
            //            Page.ClientScript.RegisterStartupScript(this.GetType(), "Initialize" + ClientID, jsInitialize, true);


            base.OnPreRender(e);
        }

        protected override void OnLoad(EventArgs e)
        {
            base.OnLoad(e);

            if (!Page.ClientScript.IsClientScriptIncludeRegistered(this.GetType(), "CoolCore.js"))
            {
                string _url = Page.ClientScript.GetWebResourceUrl(this.GetType(), "IdeaSparx.CoolControls.Web.CoolCore.js");
                Page.ClientScript.RegisterClientScriptInclude(this.GetType(), "CoolCore.js", _url);
            }

            if (!Page.ClientScript.IsClientScriptIncludeRegistered(this.GetType(), "CoolGridView.js"))
            {
                string _url = Page.ClientScript.GetWebResourceUrl(this.GetType(), "IdeaSparx.CoolControls.Web.CoolGridView.js");
                Page.ClientScript.RegisterClientScriptInclude(this.GetType(), "CoolGridView.js", _url);
            }

            //Register the DIV resize column vertical line
            if (!Page.ClientScript.IsStartupScriptRegistered(this.GetType(), "lLKAopspo28lOANcaju9182ia92u"))
            {
                Page.ClientScript.RegisterStartupScript(this.GetType(), "lLKAopspo28lOANcaju9182ia92u", "<div id=\"lLKAopspo28lOANcaju9182ia92u\" style=\"display:none;border:solid 1px gray; background-color:#E5E5E5; width:100px; height:100px; top:0px; left:0px; position:absolute;\" ></div>", false);
            }

            //if (!Page.ClientScript.IsOnSubmitStatementRegistered(this.GetType(), "CoolGridView.js.OnFormSubmit"))
            //{
            //    Page.ClientScript.RegisterOnSubmitStatement(this.GetType(), "CoolGridView.js.OnFormSubmit", String.Format(
            //        @"if (typeof var{0} != 'undefined' || var{0} != null) var{0}.OnFormSubmit();", ClientID
            //        ));
            //}

            //Register the DIV resize column vertical line
            if (!Page.ClientScript.IsClientScriptBlockRegistered(this.GetType(), "CoolGridView.Style"))
            {
                Page.ClientScript.RegisterClientScriptBlock(this.GetType(), "CoolGridView.Style",
@"<style>
    .CoolGridViewTable TR TD, .CoolGridViewTable TR TH, .CoolGridViewTable TR TH SPAN, .CoolGridViewTable TR TD SPAN
    {
        overflow: hidden;
        text-overflow : ellipsis;
    }
    .CoolGridViewTable TR TH SPAN, .CoolGridViewTable TR TD SPAN
    {
        margin : 0 0 0 0;
        padding : 0 0 0 0;
    }
</style>", false);
            }

            //Check control state from hidden field
            _HiddenFieldDataValue = Page.Request.Form[HiddenFieldDataID];
            //Load column widths from current UI state
            //Initializes the column widths base on current UI state.
            List<Unit> columnWidths = ParseColumnWidthsFromJson(_HiddenFieldDataValue);
            int cI = 0;
            foreach (DataControlField fld in Columns)
            {
                if (fld.Visible && cI < columnWidths.Count)
                    fld.HeaderStyle.Width = columnWidths[cI++];
            }
        }

        private void AddPropertiesToRenderForGridContainer(HtmlTextWriter writer)
        {
            if (Width != Unit.Empty) writer.AddStyleAttribute(HtmlTextWriterStyle.Width, Width.ToString());
            if (Height != Unit.Empty) writer.AddStyleAttribute(HtmlTextWriterStyle.Height, Height.ToString());
            //TODO:Integrate this line with main trunk
            writer.AddStyleAttribute(HtmlTextWriterStyle.Overflow, "hidden");
            BoundaryStyle.AddAttributesToRender(writer);
        }

        private string ToTableRulesValue(GridLines GridLines)
        {
            switch (GridLines)
            {
                case GridLines.Both: return "all";
                case GridLines.Horizontal: return "rows";
                case GridLines.Vertical: return "cols";
                case GridLines.None:
                default: return "none";
            }
        }

        private void RenderAttributesForTable(HtmlTextWriter writer)
        {
            //writer.AddAttribute(HtmlTextWriterAttribute.Style, Style.Value);
            //writer.AddAttribute(HtmlTextWriterAttribute.Cellspacing, CellSpacing.ToString());
            if (GridLines != GridLines.None)
            {
                writer.AddAttribute(HtmlTextWriterAttribute.Border, (BorderWidth == Unit.Empty ? "1" : BorderWidth.Value.ToString()));
                //writer.AddAttribute(HtmlTextWriterAttribute.Rules, ToTableRulesValue(GridLines));
            }
            //if (BorderWidth != Unit.Empty) writer.AddStyleAttribute(HtmlTextWriterStyle.BorderWidth, BorderWidth.ToString());
            ControlStyle.AddAttributesToRender(writer);
            writer.AddStyleAttribute("table-layout", "fixed");
            writer.AddStyleAttribute(HtmlTextWriterStyle.Display, "inline-table");
        }

        private void RenderHeader(HtmlTextWriter writer)
        {
            if (DesignMode)
            {
                if (Height != Unit.Empty) writer.AddStyleAttribute(HtmlTextWriterStyle.Height, "30px");
                if (Width != Unit.Empty) writer.AddStyleAttribute(HtmlTextWriterStyle.Width, this.Width.ToString());
            }

            writer.AddAttribute(HtmlTextWriterAttribute.Id, HeaderContainerID);
            writer.AddStyleAttribute(HtmlTextWriterStyle.OverflowX, "hidden");
            writer.AddStyleAttribute(HtmlTextWriterStyle.OverflowY, "visible");
            writer.RenderBeginTag(HtmlTextWriterTag.Div);

            //writer.AddStyleAttribute(HtmlTextWriterStyle.Width, (GetCorrectedWidth().Value + 100).ToString() + "px");
            //writer.AddStyleAttribute(HtmlTextWriterStyle.Display, "inline");
            writer.AddStyleAttribute(HtmlTextWriterStyle.BorderStyle, "none");
            writer.RenderBeginTag(HtmlTextWriterTag.Div);

            RenderAttributesForTable(writer);
            //TODO: Have to emit header specific Style
            //HeaderStyle.AddAttributesToRender(writer);
            writer.AddStyleAttribute(HtmlTextWriterStyle.Width, "0");
            writer.RenderBeginTag(HtmlTextWriterTag.Table);

            if (HeaderRow != null && HeaderRow is CoolGridViewRow)
            {
                ((CoolGridViewRow)HeaderRow).RenderColGroup(writer);
                ((CoolGridViewRow)HeaderRow).RenderRow(writer);
            }

            writer.RenderEndTag();//Table
            writer.RenderEndTag();//Div
            writer.RenderEndTag();//Div
        }

        private void RenderFooter(HtmlTextWriter writer)
        {
            if (DesignMode)
            {
                if (Height != Unit.Empty) writer.AddStyleAttribute(HtmlTextWriterStyle.Height, "30px");
                if (Width != Unit.Empty) writer.AddStyleAttribute(HtmlTextWriterStyle.Width, this.Width.ToString());
            }
            writer.AddAttribute(HtmlTextWriterAttribute.Id, FooterContainerID);
            writer.AddStyleAttribute(HtmlTextWriterStyle.OverflowX, "hidden");
            writer.AddStyleAttribute(HtmlTextWriterStyle.OverflowY, "visible");
            writer.RenderBeginTag(HtmlTextWriterTag.Div);

            //writer.AddStyleAttribute(HtmlTextWriterStyle.Width, (GetCorrectedWidth().Value + 100).ToString() + "px");
            //writer.AddStyleAttribute(HtmlTextWriterStyle.Display, "inline");
            writer.AddStyleAttribute(HtmlTextWriterStyle.BorderStyle, "none");
            writer.RenderBeginTag(HtmlTextWriterTag.Div);

            RenderAttributesForTable(writer);
            //TODO: Have to emit header specific Style
            //FooterStyle.AddAttributesToRender(writer);
            writer.AddStyleAttribute(HtmlTextWriterStyle.Width, "0");
            writer.RenderBeginTag(HtmlTextWriterTag.Table);

            if (FooterRow != null && FooterRow is CoolGridViewRow)
            {
                ((CoolGridViewRow)FooterRow).RenderColGroup(writer);
                ((CoolGridViewRow)FooterRow).RenderRow(writer);
            }

            writer.RenderEndTag();//Table
            writer.RenderEndTag();//Div
            writer.RenderEndTag();//Div
        }

        private void RenderPager(HtmlTextWriter writer, string IDSufix)
        {
            if (DesignMode)
            {
                if (Height != Unit.Empty) writer.AddStyleAttribute(HtmlTextWriterStyle.Height, "30px");
                if (Width != Unit.Empty) writer.AddStyleAttribute(HtmlTextWriterStyle.Width, this.Width.ToString());
            }
            writer.AddAttribute(HtmlTextWriterAttribute.Id, PagerContainerID + IDSufix);
            writer.AddStyleAttribute(HtmlTextWriterStyle.OverflowX, "hidden");
            writer.AddStyleAttribute(HtmlTextWriterStyle.OverflowY, "visible");
            writer.RenderBeginTag(HtmlTextWriterTag.Div);

            PagerStyle.AddAttributesToRender(writer);
            //writer.AddStyleAttribute(HtmlTextWriterStyle.Width, GetCorrectedWidth().ToString());
            writer.RenderBeginTag(HtmlTextWriterTag.Table);

            if (AllowPaging && PagerRow != null)
            {
                ((CoolGridViewRow)PagerRow).RenderRow(writer);
            }

            writer.RenderEndTag();//Table
            writer.RenderEndTag();//Div
        }

        protected override void Render(HtmlTextWriter writer)
        {
            String orig_CssClass = CssClass;
            try
            {
                CssClass = (String.IsNullOrEmpty(this.CssClass) ? "CoolGridViewTable" : "CoolGridViewTable " + CssClass);
                Style["table-layout"] = "fixed";
                Style[HtmlTextWriterStyle.Display] = "table";
                Style[HtmlTextWriterStyle.BorderCollapse] = "collapse";
                //base.Width = GetCorrectedWidth();
                base.Width = new Unit(0, UnitType.Pixel);
                base.Height = Unit.Empty;

                //from base Render
                if (this.Page != null)
                    this.Page.VerifyRenderingInServerForm(this);
                //from base Render
                this.PrepareControlHierarchy();

                writer.AddAttribute(HtmlTextWriterAttribute.Id, GridContainerID);
                AddPropertiesToRenderForGridContainer(writer);
                writer.RenderBeginTag(HtmlTextWriterTag.Div); //Div 1

                //Render the Pager on Top
                if (AllowPaging && (this.PagerSettings.Position == PagerPosition.Top || this.PagerSettings.Position == PagerPosition.TopAndBottom) && this.PagerRow != null)
                    RenderPager(writer, "Top");

                //Render the fixed header
                if (ShowHeader && Rows.Count > 0)
                    RenderHeader(writer);

                if (DesignMode)
                {
                    if (Height != Unit.Empty)
                    {
                        double _height = Height.Value;
                        if (ShowHeader) _height -= 30;
                        if (ShowFooter) _height -= 30;
                        if (AllowPaging) _height -= 30;
                        writer.AddStyleAttribute(HtmlTextWriterStyle.Height, _height.ToString() + "px");
                    }
                    if (Width != Unit.Empty) writer.AddStyleAttribute(HtmlTextWriterStyle.Width, this.Width.ToString());
                }
                writer.AddAttribute(HtmlTextWriterAttribute.Id, TableContainerID);
                writer.AddStyleAttribute(HtmlTextWriterStyle.Overflow, "auto");
                //Compatibility for IE 6.0
                writer.AddStyleAttribute(HtmlTextWriterStyle.Width, "100%");
                writer.RenderBeginTag(HtmlTextWriterTag.Div); //Div 2
                base.RenderContents(writer);
                writer.RenderEndTag();//Div 2

                if (ShowFooter)
                    RenderFooter(writer);
                //Render the pager at bottom
                if (AllowPaging && (this.PagerSettings.Position == PagerPosition.Bottom || this.PagerSettings.Position == PagerPosition.TopAndBottom) && this.PagerRow != null)
                    RenderPager(writer, "Bottom");

                writer.RenderEndTag();//Div 1

                string jsInitialize = @"<input type=""hidden"" id=""{6}"" name=""{6}"" value=""{7}"" /><script type=""text/javascript"" language=""javascript"">
//<![CDATA[
if (typeof var{3} =='undefined' || var{3} == null)
    var var{3} = new CoolGridView({{GridContainerID: ""{0}"",HeaderContainerID: ""{1}"",TableContainerID: ""{2}"",GridID: ""{3}"",FooterContainerID: ""{4}"",PagerContainerID: ""{5}"",HiddenFieldDataID: ""{6}"",FormID:""{8}"", AllowResizeColumn : {9}}});
var{3}.Initialize();
//]]>
</script>";

                //Register JavaScript Initialization
                jsInitialize = String.Format(
                    jsInitialize,
                    GridContainerID,
                    HeaderContainerID,
                    TableContainerID,
                    ClientID,
                    FooterContainerID,
                    PagerContainerID,
                    HiddenFieldDataID,
                    _HiddenFieldDataValue,
                    this.Page.Form.ClientID,
                    AllowResizeColumn.ToString().ToLower()
                    );
                writer.Write(jsInitialize);
            }
            catch (Exception ex)
            {
                writer.Write(ex.Message);
            }
            finally
            {
                CssClass = orig_CssClass;
            }
        }
    }

    /// <summary>
    /// The base CoolGridViewRow
    /// </summary>
    public class BaseCoolGridViewRow : GridViewRow
    {
        //REgular expression for inserting an inner span in <TD></TD>
        private Regex _reg;
        private CoolGridView _ParentCoolGridView = null;
        /// <summary>
        /// Gets the parent CoolGridView
        /// </summary>
        internal CoolGridView ParentCoolGridView
        {
            get {
                if (_ParentCoolGridView == null)
                {
                    Control c = this;
                    while((c = c.Parent) != null){
                        if (c is CoolGridView)
                        {
                            _ParentCoolGridView = (CoolGridView)c;
                            break;
                        }
                    }
                }
                return _ParentCoolGridView; 
            }
            set
            {
                _ParentCoolGridView = value;
            }
        }
        /// <summary>
        /// Initializes a new instance of the System.Web.UI.WebControls.GridViewRow class.
        /// </summary>
        /// <param name="rowIndex">The index of the System.Web.UI.WebControls.GridViewRow object in the System.Web.UI.WebControls.GridView.Rows
        ///     collection of a System.Web.UI.WebControls.GridView control.</param>
        /// <param name="dataItemIndex">The index of the System.Web.UI.WebControls.GridViewRow object in the System.Web.UI.WebControls.GridView.Rows
        ///     collection of a System.Web.UI.WebControls.GridView control.</param>
        /// <param name="rowType">One of the System.Web.UI.WebControls.DataControlRowType enumeration values.</param>
        /// <param name="rowState">A bitwise combination of the System.Web.UI.WebControls.DataControlRowState
        ///     enumeration values.</param>
        public BaseCoolGridViewRow(int rowIndex, int dataItemIndex, DataControlRowType rowType, DataControlRowState rowState)
            : base(rowIndex, dataItemIndex, rowType, rowState)
        {
            string regex = "(^\\s*<(?:td|th)[^>]*?>)(.*)(</(?:td|th)>\\s*$)";
            RegexOptions options = RegexOptions.IgnorePatternWhitespace | RegexOptions.Singleline | RegexOptions.Multiline | RegexOptions.IgnoreCase;
            _reg = new Regex(regex, options);
        }

        /// <summary>
        /// Render a control and return the resulting HTML
        /// </summary>
        /// <param name="control">Control to render</param>
        /// <returns>String that contains the rendered HTML</returns>
        protected string GetRendering(Control control)
        {
            using (StringWriter sw = new StringWriter())
            using (HtmlTextWriter htmlw = new HtmlTextWriter(sw))
            {
                control.RenderControl(htmlw);
                return sw.ToString();
            }
        }

        /// <summary>
        /// Place a SPAN just inside TD
        /// </summary>
        /// <param name="Html">The html of the table cell</param>
        /// <returns>returns the resulting string HTML with span inserted inside, enclosing the TD's content</returns>
        protected string PutInsideSpan(string Html)
        {
            if (String.IsNullOrEmpty(Html))
                return Html;

            Match m = _reg.Match(Html);
            if (m != null && m.Groups.Count >= 3)
                return m.Groups[1].Captures[0].Value + "<span>" + m.Groups[2].Captures[0].Value + "</span>" + m.Groups[3].Captures[0].Value;

            return Html;
        }

        internal virtual void RenderColGroup(HtmlTextWriter writer)
        {
            //foreach (DataControlFieldCell cell in Cells)
            //    cell.Width = GetCorrectedCellWidth(cell);

            writer.RenderBeginTag(HtmlTextWriterTag.Colgroup);

            if (ParentCoolGridView.ViewMode == CoolGridViewViewMode.Hierarchical)
            {
                writer.Write("<col width=\"20px\" />");
            }

            foreach (DataControlFieldCell cell in Cells)
            {
                if (cell.ContainingField.Visible)
                {
                    writer.Write("<col width=\"{0}\" {1} />",
                        GetCorrectedCellWidth(cell).Value.ToString(),
                        (String.IsNullOrEmpty(cell.ContainingField.HeaderStyle.CssClass) ? String.Empty : "class=\"" + cell.ContainingField.HeaderStyle.CssClass + "\"")
                        );
                }
            }
            writer.RenderEndTag();
        }

        protected virtual Unit GetCorrectedCellWidth(DataControlFieldCell cell)
        {
            switch (RowType)
            {
                case DataControlRowType.Header:
                case DataControlRowType.Footer:
                    if (cell.ContainingField.HeaderStyle.Width == Unit.Empty || cell.ContainingField.HeaderStyle.Width.Type == UnitType.Percentage)
                    {
                        return ParentCoolGridView.DefaultColumnWidth;
                    }
                    else
                    {
                        return cell.ContainingField.HeaderStyle.Width;
                    }
                default:
                    return Width;
            }
        }

        internal virtual void RenderRow(HtmlTextWriter writer)
        {
            base.Render(writer);
        }

        protected override void RenderContents(HtmlTextWriter writer)
        {
            if (ParentCoolGridView.ViewMode == CoolGridViewViewMode.Hierarchical)
            {
                switch (RowType)
                {
                    case DataControlRowType.DataRow:
                        writer.Write("<td>+</td>");
                        break;
                    case DataControlRowType.Header:
                        writer.Write("<th></th>");
                        break;
                    case DataControlRowType.Footer:
                        writer.Write("<td></td>");
                        break;
                }
                
            }
            base.RenderContents(writer);
        }
    }

    /// <summary>
    /// CoolGridViewRow represents Header, Footer or Pager of CoolGridView
    /// </summary>
    public class CoolGridViewRow : BaseCoolGridViewRow
    {
        public CoolGridViewRow(int rowIndex, int dataItemIndex, DataControlRowType rowType, DataControlRowState rowState)
            : base(rowIndex, dataItemIndex, rowType, rowState)
        {
        }

        protected override void Render(HtmlTextWriter writer)
        {
            switch (RowType)
            {
                case DataControlRowType.Header:
                case DataControlRowType.Footer:
                    break;
            }
        }

        ////There seems to be a bug in IE 6 and IE 7 when rendering TD with defined width.
        ////The style white-space nowrap is not taking effect.
        ////The fix is to put an inner span just inside <TD></TD> and put the "white-space:nospace" stye on the span.
        ///// <summary>
        ///// 
        ///// </summary>
        ///// <param name="writer"></param>
        //protected override void RenderContents(HtmlTextWriter writer)
        //{
        //    if (RowType != DataControlRowType.DataRow)
        //        foreach (Control control in Controls)
        //            writer.Write(PutInsideSpan(GetRendering(control)));
        //    else
        //        base.RenderContents(writer);
        //}
    }

    public class BaseCoolGridViewDataRow : BaseCoolGridViewRow
    {
        private bool _IsChildExpanded = false;
        /// <summary>
        /// Gets or sets if the child control is expanded or not.
        /// </summary>
        public bool IsChildExpanded
        {
            get { return _IsChildExpanded; }
            set { _IsChildExpanded = value; }
        }

        public BaseCoolGridViewDataRow(int rowIndex, int dataItemIndex, DataControlRowType rowType, DataControlRowState rowState)
            : base(rowIndex, dataItemIndex, rowType, rowState)
        {
            this.ID = String.Format("tr%d{0}%r{1}",
                    dataItemIndex,
                    rowIndex
                );
        }

        protected override void RenderContents(HtmlTextWriter writer)
        {
            base.RenderContents(writer);

            if (!this.DesignMode)
            {
                writer.Write(
                    String.Format("</tr><tr{0}><td colspan=\"0\" style=\"overflow:visible;padding:0 0 0 0;\"><div>",
                    (IsChildExpanded ? String.Empty : " style=\"display:none;\"")));
                if (this.ParentCoolGridView._ChildInstance != null)
                {
                    this.ParentCoolGridView._ChildInstance.ID = this.ID;
                    this.ParentCoolGridView._ChildInstance.CustomRenderControl(writer);
                }
                writer.Write("</div></td>");
                //writer.Write("</tr>");
            }
        }
    }

    public class CoolGridViewDataRow : BaseCoolGridViewDataRow
    {
        public CoolGridViewDataRow(int rowIndex, int dataItemIndex, DataControlRowType rowType, DataControlRowState rowState)
            : base(rowIndex, dataItemIndex, rowType, rowState)
        {
        }

    }

    /// <summary>
    /// CoolGridViewFirstRow is the first row of CoolGridView. It is the header if header is visible, or the first datarow if header is not visible.
    /// * Changed CoolGridViewFirstRow to CoolGridViewFirstDataRow.
    /// </summary>
    public class CoolGridViewFirstDataRow : BaseCoolGridViewDataRow
    {
        public CoolGridViewFirstDataRow(int rowIndex, int dataItemIndex, DataControlRowType rowType, DataControlRowState rowState)
            : base(rowIndex, dataItemIndex, rowType, rowState)
        {           
        }

        protected override void Render(HtmlTextWriter writer)
        {
            RenderColGroup(writer);
            base.Render(writer);
        }

        ////There seems to be a bug in IE 6 and IE 7 when rendering TD with defined width.
        ////The style white-space nowrap is not taking effect.
        ////The fix is to put an inner span just inside <TD></TD> and put the "white-space:nospace" stye on the span.
        ///// <summary>
        ///// 
        ///// </summary>
        ///// <param name="writer"></param>
        //protected override void RenderContents(HtmlTextWriter writer)
        //{
        //    foreach (Control control in Controls)
        //        writer.Write(PutInsideSpan(GetRendering(control)));
        //}
        protected override Unit GetCorrectedCellWidth(DataControlFieldCell cell)
        {
            if (cell.ContainingField.HeaderStyle.Width == Unit.Empty || cell.ContainingField.HeaderStyle.Width.Type == UnitType.Percentage)
            {
                return ParentCoolGridView.DefaultColumnWidth;
            }
            else
            {
                return cell.ContainingField.HeaderStyle.Width;
            }
        }
    }
}
